@@ -168,7 +168,6 @@ free_selected_file_array(FileView *view)
 			my_free(view->selected_filelist);
 		view->selected_filelist = NULL;
 	}
-	view->selected_filelist = NULL;
 }
 
 /* If you use this function using the free_selected_file_array() 
@@ -474,7 +473,6 @@ moveto_list_pos(FileView *view, int pos)
 	char file_name[view->window_width +2];
 	int x;
 
-
 	if(pos < 1)
 		pos = 0;
 
@@ -511,6 +509,7 @@ moveto_list_pos(FileView *view, int pos)
 	view->list_pos = pos;
 
 
+
 	if(redraw)
 		draw_dir_list(view, view->top_line, view->curr_line);
 
@@ -528,6 +527,7 @@ moveto_list_pos(FileView *view, int pos)
 
 			for (x = strlen(file_name); x < view->window_width ; x++)
 				file_name[x] = ' ';
+
 			file_name[view->window_width] = ' ';
 			file_name[view->window_width +1 ] = '\0';
 			mvwaddstr(view->win, view->curr_line, 0, file_name);
@@ -792,7 +792,6 @@ change_directory(FileView *view, char *directory)
 	stat(view->curr_dir, &s);
 	view->dir_mtime = s.st_mtime;
 	closedir(dir);
-	update_term_title(view->curr_dir);
 }
 
 static void
@@ -822,9 +821,6 @@ load_dir_list(FileView *view, int reload)
 	int namelen = 0;
 	int old_list = view->list_rows;
 
-
-	curr_stats.block_timer_thread = 1;
-
 	dir = opendir(view->curr_dir);
 
 	if(dir == NULL)
@@ -972,6 +968,10 @@ load_dir_list(FileView *view, int reload)
 
 	closedir(dir);
 	
+	if(!reload && s.st_size > 2048)
+	{
+		status_bar_message("Sorting Directory...");
+	}
 	qsort(view->dir_entry, view->list_rows, sizeof(dir_entry_t),
 				sort_dir_list);
 
@@ -1017,7 +1017,6 @@ load_dir_list(FileView *view, int reload)
 
 	draw_dir_list(view, view->top_line, view->list_pos);
 
-	curr_stats.block_timer_thread = 0;
 	return;
 }
 
