@@ -44,7 +44,7 @@
 
 
 static void
-add_sort_type_info(FileView *view, int y, int x)
+add_sort_type_info(FileView *view, int y, int x, int current_line)
 {
 	char buf[24];
 	struct passwd *pwd_buf;
@@ -120,8 +120,28 @@ add_sort_type_info(FileView *view, int y, int x)
 				snprintf(buf, sizeof(buf), " %d", view->dir_entry[x].size);
 			break;
     }
+	if (cfg.use_color)
+	{
+		if (current_line)
+			wattron(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
+
 		mvwaddstr(view->win, y, 
 					view->window_width - strlen(buf), buf); 
+
+		if (current_line)
+			wattroff(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
+	}
+	else
+	{
+		if (current_line)
+			wattron(view->win, A_REVERSE);
+
+		mvwaddstr(view->win, y, 
+					view->window_width - strlen(buf), buf); 
+
+	 	if (current_line)
+			wattroff(view->win, A_REVERSE);
+	}
 }
 
 char *
@@ -140,15 +160,14 @@ free_selected_file_array(FileView *view)
 		{
 			if(view->selected_filelist[x])
 			{
-				free(view->selected_filelist[x]);
+				my_free(view->selected_filelist[x]);
 			}
 
 		}
 		if(view->selected_filelist)
-			free(view->selected_filelist);
+			my_free(view->selected_filelist);
 		view->selected_filelist = NULL;
 	}
-	view->selected_filelist = NULL;
 }
 
 /* If you use this function using the free_selected_file_array() 
@@ -224,7 +243,7 @@ draw_dir_list(FileView *view, int top, int pos)
 
 	werase(view->win);
 	werase(view->title);
-	wprintw(view->title, view->curr_dir);
+	wprintw(view->title, "%s", view->curr_dir);
 	wnoutrefresh(view->title);
 
 		/* This is needed for reloading a list that has had files deleted */
@@ -253,7 +272,7 @@ draw_dir_list(FileView *view, int top, int pos)
 			strncpy(file_name,  view->dir_entry[x].name, sizeof(file_name) -1);
 			file_name[view->window_width -2] = '\0';
 
-			wmove(view->win, y, 2);
+			wmove(view->win, y, 1);
 			if(view->dir_entry[x].selected)
 			{
 				LINE_COLOR = SELECTED_COLOR;
@@ -287,18 +306,18 @@ draw_dir_list(FileView *view, int top, int pos)
 			{
 				wattrset(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
 				wattron(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
-				wprintw(view->win, file_name);
+				wprintw(view->win, "%s", file_name);
 				wattroff(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
 
-				add_sort_type_info(view, y, x);
+				add_sort_type_info(view, y, x, 0);
 			}
 			else
 			{
 				wattrset(view->win, COLOR_PAIR(LINE_COLOR));
 				wattron(view->win, COLOR_PAIR(LINE_COLOR));
-				wprintw(view->win, file_name);
+				wprintw(view->win, "%s", file_name);
 				wattroff(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
-				add_sort_type_info(view, y, x);
+				add_sort_type_info(view, y, x, 0);
 				bold = 1;
 			}
 			y++;
@@ -318,19 +337,19 @@ draw_dir_list(FileView *view, int top, int pos)
 				wattrset(view->win, A_REVERSE);
 				wattron(view->win, A_REVERSE);
 			}
-			wmove(view->win, y, 2);
+			wmove(view->win, y, 1);
 
-			wprintw(view->win, file_name);
+			wprintw(view->win, "%s", file_name);
 			wattroff(view->win, A_BOLD);
 			wattroff(view->win, A_REVERSE);
-			add_sort_type_info(view, y, x);
+			add_sort_type_info(view, y, x, 0);
 			y++;
 			if(y > view->window_rows)
 				break;
 		}
 	}
 	if(view != curr_view)
-		mvwaddstr(view->win, view->curr_line, 0, " *");
+		mvwaddstr(view->win, view->curr_line, 0, "*");
 
 	view->top_line = top;
 }
@@ -344,14 +363,115 @@ S_ISEXE(mode_t mode)
 	return 0;
 }
 
+void
+erase_current_line_bar(FileView *view)
+{
+	int old_cursor = view->curr_line;
+	int old_pos = view->top_line + old_cursor;
+	char file_name[view->window_width -2];
+	int bold = 1;
+
+	if(cfg.use_color)
+	{
+		int LINE_COLOR;
+
+		/* Extra long file names are truncated to fit */
+
+		wattroff(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
+		if((old_pos > -1)  && (old_pos < view->list_rows))
+		{
+			strncpy(file_name,  view->dir_entry[old_pos].name, 
+					sizeof(file_name) -2);
+		}
+		else /* The entire list is going to be redrawn so just return. */
+			return;
 
 
 
+		wmove(view->win, old_cursor, 1);
+
+		wclrtoeol(view->win);
+
+		if(view->dir_entry[old_pos].selected)
+		{
+			LINE_COLOR = SELECTED_COLOR;
+		}
+		else
+		{
+			switch(view->dir_entry[old_pos].type)
+			{
+				case DIRECTORY:
+					LINE_COLOR = DIRECTORY_COLOR;
+					break;
+				case LINK:
+					LINE_COLOR = LINK_COLOR;
+					break;
+				case SOCKET:
+					LINE_COLOR = SOCKET_COLOR;
+					break;
+				case DEVICE:
+					LINE_COLOR = DEVICE_COLOR;
+					break;
+				case EXECUTABLE:
+					LINE_COLOR = EXECUTABLE_COLOR;
+					break;
+				default:
+					LINE_COLOR = WIN_COLOR;
+					bold = 0;
+					break;
+			}
+		}
+		if(bold)
+		{
+			wattrset(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
+			wattron(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
+			mvwaddnstr(view->win, old_cursor, 1, file_name, view->window_width -4);
+			wattroff(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
+
+			add_sort_type_info(view, old_cursor, old_pos, 0);
+		}
+		else
+		{
+			wattrset(view->win, COLOR_PAIR(LINE_COLOR));
+			wattron(view->win, COLOR_PAIR(LINE_COLOR));
+
+			mvwaddnstr(view->win, old_cursor, 1, file_name, view->window_width -4);
+			wattroff(view->win, COLOR_PAIR(LINE_COLOR) | A_BOLD);
+			add_sort_type_info(view, old_cursor, old_pos, 0);
+			bold = 1;
+		}
+	}
+	else
+	{
+		wattroff(view->win, A_BOLD);
+		wattroff(view->win, A_REVERSE);
+
+		strncpy(file_name,  view->dir_entry[old_pos].name, 
+					sizeof(file_name) -2);
+
+		wmove(view->win, old_cursor, 1);
+		wclrtoeol(view->win);
+
+		if(view->dir_entry[old_pos].selected)
+		{
+			wattrset(view->win, A_BOLD);
+			wattron(view->win, A_BOLD);
+		}
+		wmove(view->win, view->curr_line, 1);
+		wprintw(view->win, "%s", file_name);
+		wattroff(view->win, A_BOLD);
+		wattroff(view->win, A_REVERSE);
+		add_sort_type_info(view, old_cursor, old_pos, 0);
+	}
+}
+
 void
 moveto_list_pos(FileView *view, int pos)
 {
 	int redraw = 0;
 	int old_cursor = view->curr_line;
+	char file_name[view->window_width +2];
+	int x;
 
 	if(pos < 1)
 		pos = 0;
@@ -359,6 +479,12 @@ moveto_list_pos(FileView *view, int pos)
 	if(pos > view->list_rows -1)
 		pos = (view->list_rows -1);
 
+	
+	if(view->curr_line > view->list_rows -1)
+		view->curr_line = view->list_rows -1;
+
+	erase_current_line_bar(view);
+
 	if((view->top_line <=  pos) && (pos <= (view->top_line + view->window_rows)))
 	{
 		view->curr_line = pos - view->top_line;
@@ -383,24 +509,81 @@ moveto_list_pos(FileView *view, int pos)
 	view->list_pos = pos;
 
 
+
 	if(redraw)
 		draw_dir_list(view, view->top_line, view->curr_line);
 
-
 	if(old_cursor != view->curr_line)
 	{
 		wattroff(view->win, COLOR_PAIR(CURR_LINE_COLOR));
-		mvwaddstr(view->win, old_cursor, 0, "  ");
+		mvwaddstr(view->win, old_cursor, 0, " ");
 
-		wattron(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
-		mvwaddstr(view->win, view->curr_line, 0, "=>");
+		if(cfg.use_color)
+		{
+
+			snprintf(file_name, sizeof(file_name) -1, " %s", 
+					view->dir_entry[pos].name);
+			wattron(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
+
+			for (x = strlen(file_name); x < view->window_width ; x++)
+				file_name[x] = ' ';
+
+			file_name[view->window_width] = ' ';
+			file_name[view->window_width +1 ] = '\0';
+			mvwaddstr(view->win, view->curr_line, 0, file_name);
+
+			add_sort_type_info(view, view->curr_line, pos, 1);
+		}
+		else /* Not using color */
+		{
+			wattroff(view->win, A_BOLD);
+
+			snprintf(file_name, sizeof(file_name) -1, " %s", 
+					view->dir_entry[pos].name);
+
+			wattron(view->win,  A_REVERSE);
+
+			for (x = strlen(file_name); x < view->window_width ; x++)
+				file_name[x] = ' ';
+			file_name[view->window_width] = ' ';
+			file_name[view->window_width +1 ] = '\0';
+			mvwaddstr(view->win, view->curr_line, 0, file_name);
+			add_sort_type_info(view, view->curr_line, pos, 1);
+		}
 	}
 	else
 	{
-		wattron(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
-		mvwaddstr(view->win, view->curr_line, 0, "=>");
+		if (cfg.use_color)
+		{
+			int x;
+			snprintf(file_name, sizeof(file_name), " %s", 
+						view->dir_entry[pos].name);
+			wattron(view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
+			for (x = strlen(file_name); x < view->window_width; x++)
+				file_name[x] = ' ';
+			file_name[view->window_width] = ' ';
+			file_name[view->window_width +1] = '\0';
+			mvwaddstr(view->win, view->curr_line, 0, file_name);
+
+			add_sort_type_info(view, view->curr_line, pos, 1);
+		}
+		else
+		{
+			int x;
+			snprintf(file_name, sizeof(file_name), " %s", 
+						view->dir_entry[pos].name);
+			wattrset(view->win, A_REVERSE);
+			wattron(view->win,  A_REVERSE);
+			for (x = strlen(file_name); x < view->window_width; x++)
+				file_name[x] = ' ';
+			file_name[view->window_width] = ' ';
+			file_name[view->window_width +1] = '\0';
+			mvwaddstr(view->win, view->curr_line, 0, file_name);
+			wattroff(view->win, A_REVERSE);
+
+			add_sort_type_info(view, view->curr_line, pos, 1);
+		}
 	}
-	
 }
 
 
@@ -442,7 +625,7 @@ save_view_history(FileView *view)
 	if(found)
 	{
 		snprintf(view->history[x].file, sizeof(view->history[x].file),
-				view->dir_entry[view->list_pos].name);
+				"%s", view->dir_entry[view->list_pos].name);
 		return;
 	}
 
@@ -452,23 +635,23 @@ save_view_history(FileView *view)
 		for(y = 0; y < cfg.history_len -1; y++)
 		{
 			snprintf(view->history[y].file, sizeof(view->history[y].file),
-						view->history[y +1].file);
+						"%s", view->history[y +1].file);
 			snprintf(view->history[y].dir, sizeof(view->history[y].dir),
-					view->history[y +1].dir);
+					"%s", view->history[y +1].dir);
 		}
 		snprintf(view->history[cfg.history_len -1].file, 
 				sizeof(view->history[cfg.history_len -1].file),
-				view->dir_entry[view->list_pos].name);
+				"%s", view->dir_entry[view->list_pos].name);
 		snprintf(view->history[cfg.history_len -1].dir, 
 				sizeof(view->history[cfg.history_len -1].dir),
-				view->curr_dir);
+				"%s", view->curr_dir);
 	}
 	else
 	{
 		snprintf(view->history[x].dir, sizeof(view->history[x].dir),
-			view->curr_dir);
+			"%s", view->curr_dir);
 		snprintf(view->history[x].file, sizeof(view->history[x].file),
-				view->dir_entry[view->list_pos].name);
+				"%s", view->dir_entry[view->list_pos].name);
 		view->history_num++;
 	}
 	return;
@@ -598,7 +781,7 @@ change_directory(FileView *view, char *directory)
 		return;
 	}
 
-	snprintf(view->last_dir, sizeof(view->last_dir), view->curr_dir);
+	snprintf(view->last_dir, sizeof(view->last_dir), "%s", view->curr_dir);
 
 	clean_selected_files(view);
 		
@@ -609,7 +792,6 @@ change_directory(FileView *view, char *directory)
 	stat(view->curr_dir, &s);
 	view->dir_mtime = s.st_mtime;
 	closedir(dir);
-	update_term_title(view->curr_dir);
 }
 
 static void
@@ -628,6 +810,7 @@ reset_selected_files(FileView *view)
 	free_selected_file_array(view);
 }
 
+
 void
 load_dir_list(FileView *view, int reload)
 {
@@ -638,7 +821,6 @@ load_dir_list(FileView *view, int reload)
 	int namelen = 0;
 	int old_list = view->list_rows;
 
-
 	dir = opendir(view->curr_dir);
 
 	if(dir == NULL)
@@ -654,8 +836,6 @@ load_dir_list(FileView *view, int reload)
 	{
 		status_bar_message("Reading Directory...");
 	}
-	else
-		status_bar_message("");
 
 	update_all_windows();
 
@@ -788,6 +968,10 @@ load_dir_list(FileView *view, int reload)
 
 	closedir(dir);
 	
+	if(!reload && s.st_size > 2048)
+	{
+		status_bar_message("Sorting Directory...");
+	}
 	qsort(view->dir_entry, view->list_rows, sizeof(dir_entry_t),
 				sort_dir_list);
 
@@ -832,6 +1016,7 @@ load_dir_list(FileView *view, int reload)
 		reset_selected_files(view);
 
 	draw_dir_list(view, view->top_line, view->list_pos);
+
 	return;
 }
 
