@@ -24,6 +24,7 @@
 #include<sys/types.h> /* select() */
 #include<unistd.h> /* select() */
 
+#include "background.h"
 #include "bookmarks.h"
 #include "commands.h"
 #include "config.h"
@@ -32,6 +33,7 @@
 #include "fileops.h"
 #include "keys.h"
 #include "menus.h"
+#include "registers.h"
 #include "search.h"
 #include "signals.h"
 #include "sort.h"
@@ -40,6 +42,13 @@
 #include "utils.h"
 #include "visual.h"
 
+void
+switch_views(void)
+{
+	FileView *tmp = curr_view;
+	curr_view = other_view;
+	other_view = tmp;
+}
 
 void
 clean_status_bar(FileView *view)
@@ -63,6 +72,58 @@ clear_num_window(void)
 	wrefresh(num_win);
 }
 
+static void
+reload_window(FileView *view)
+{
+	struct stat s;
+
+	stat(view->curr_dir, &s);
+	if(view != curr_view)
+		change_directory(view, view->curr_dir);
+
+	load_dir_list(view, 1);
+	view->dir_mtime = s.st_mtime;
+
+	if(view != curr_view)
+	{
+		change_directory(curr_view, curr_view->curr_dir);
+		mvwaddstr(view->win, view->curr_line, 0, "*");
+		wrefresh(view->win);
+	}
+	else
+		moveto_list_pos(view, view->list_pos);
+
+}
+/*
+ * This checks the modified times of the directories.
+ */
+static void
+check_if_filelists_have_changed(FileView *view)
+{
+	struct stat s;
+
+	stat(view->curr_dir, &s);
+	if(s.st_mtime  != view->dir_mtime)
+		reload_window(view);
+
+	if (curr_stats.number_of_windows != 1)
+	{
+		stat(other_view->curr_dir, &s);
+		if(s.st_mtime != other_view->dir_mtime)
+			reload_window(other_view);
+	}
+
+}
+
+static void
+repeat_last_command(FileView *view)
+{
+	if (0 > cfg.cmd_history_num)
+		show_error_msg(" Command Error ", "Command history list is empty. ");
+	else
+		execute_command(view, cfg.cmd_history[0]);
+}
+
 
 void
 rename_file(FileView *view)
@@ -70,20 +131,25 @@ rename_file(FileView *view)
 	char * filename = get_current_file_name(view);
 	char command[1024];
 	int key;
-	int pos;
-	int index = 0;
+	int pos = strlen(filename) + 1;
+	int index = pos - 1;
 	int done = 0;
 	int abort = 0;
-	int len = 0;
+	int len = pos;
 	int found = -1;
 	char buf[view->window_width -2];
 
 	wattroff(view->win, COLOR_PAIR(CURR_LINE_COLOR));
-	pos = 1;
 	wmove(view->win, view->curr_line, 0);
 	wclrtoeol(view->win);
-
 	wmove(view->win, view->curr_line, 1);
+	waddstr(view->win, filename);
+	memset(buf, '\0', view->window_width -2);
+	strncpy(buf, filename, sizeof(buf));
+	len = strlen(filename);
+	wmove(view->win, view->curr_line, strlen(filename) + 1);
+
+//	wmove(view->win, view->curr_line, 1);
 	curs_set(1);
 
   while(!done)
@@ -112,17 +178,52 @@ rename_file(FileView *view)
 			case 8: /* ascii Backspace  ascii Ctrl H */
 			case KEY_BACKSPACE: /* ncurses BACKSPACE KEY */
 				{
+					/*
+					if(index == len)
+					{
+					*/
+						pos--;
+						index--;
+						len--;
+
+						if(pos < 1)
+							pos = 1;
+						if(index < 0)
+							index = 0;
+						
+						mvwdelch(view->win, view->curr_line, pos);
+						buf[index] = '\0';
+						buf[len] = '\0';
+				//	}
+				}
+				break;
+			case KEY_LEFT:
+				{
+					index--;
 					pos--;
-					index--;
-					len--;
 
-					if(pos < 2)
-						pos = 2;
 					if(index < 0)
 						index = 0;
-					
-					mvwdelch(view->win, view->curr_line, pos);
-					buf[index] = '\0';
+
+					if(pos < 1)
+						pos = 1;
+
+					wmove(view->win, view->curr_line, pos);
+
+				}
+				break;
+			case KEY_RIGHT:
+				{
+					index++;
+					pos++;
+
+					if(index > len)
+						index = len;
+
+					if(pos > len + 1)
+						pos = len + 1;
+
+					wmove(view->win, view->curr_line, pos);
 				}
 				break;
 			default:
@@ -132,6 +233,11 @@ rename_file(FileView *view)
 					buf[index] = key;
 					index++;
 					buf[index] = '\0';
+					if(len < index)
+					{
+						len++;
+						buf[index] = '\0';
+					}
 					if(index > 62)
 					{
 						abort = 1;
@@ -154,7 +260,7 @@ rename_file(FileView *view)
 		return;
 	}
 
-	if (access(buf, F_OK) == 0)
+	if (access(buf, F_OK) == 0 && strncmp(filename, buf, len) != 0)
 	{
 		show_error_msg("File exists", "That file already exists. Will not overwrite.");
 
@@ -241,7 +347,10 @@ yank_files(FileView *view, int count, char *count_buf)
 		{
 			view->dir_entry[y].selected = 1;
 			y++;
+			if (y >= view->list_rows)
+				break;
 		}
+		view->selected_files = y - view->list_pos;
 	}
 	else if(!view->selected_files)
 	{
@@ -260,11 +369,7 @@ yank_files(FileView *view, int count, char *count_buf)
 	view->selected_files = 0;
 
 	draw_dir_list(view, view->top_line, view->list_pos); 
-	wattron(view->win, COLOR_PAIR(CURR_LINE_COLOR)| A_BOLD);
-	mvwaddstr(view->win, view->curr_line, 0, ">");
-	wattroff(view->win, COLOR_PAIR(CURR_LINE_COLOR));
-	wmove(view->win, view->curr_line, 0);
-
+	moveto_list_pos(view, view->list_pos);
 	snprintf(buf, sizeof(buf), " %d %s yanked.", count, 
 			count == 1 ? "file" : "files");
 	status_bar_message(buf);
@@ -295,20 +400,77 @@ tag_file(FileView *view)
 		wmove(view->win, view->curr_line, 0);
 }
 
+int
+put_files_from_register(FileView *view)
+{
+	int x;
+	int i = -1;
+	int y = 0;
+	char buf[PATH_MAX + (NAME_MAX * 2) + 4];
+
+	for (x = 0; x < NUM_REGISTERS; x++)
+	{
+		if (reg[x].name == curr_stats.curr_register)
+		{
+			i = x;
+			break;
+		}
+	}
+
+	if ((i < 0) || (reg[i].num_files < 1))
+	{
+		status_bar_message("Register is empty");
+		wrefresh(status_bar);
+		return 1;
+	}
+
+	for (x = 0; x < reg[i].num_files; x++)
+	{
+		snprintf(buf, sizeof(buf), "%s/%s", cfg.trash_dir, reg[i].files[x]);
+		if (!access(buf, F_OK))
+		{
+			snprintf(buf, sizeof(buf), "mv \"%s/%s\" %s",
+					cfg.trash_dir, reg[i].files[x], view->curr_dir);
+			if ( background_and_wait_for_errors(buf))
+				y++;
+		}
+	}
+
+	clear_register(curr_stats.curr_register);
+	curr_stats.use_register  = 0;
+	curr_stats.register_saved = 0;
+
+	if (y)
+	{
+		snprintf(buf, sizeof(buf), " %d %s inserted", y,
+				y==1 ? "file" : "files");
+
+		load_dir_list(view, 0);
+		moveto_list_pos(view, view->curr_line);
+
+		status_bar_message(buf);
+		return 1;
+	}
+
+	return 0;
+}
+
 int 
 put_files(FileView *view)
 {
 	char command[NAME_MAX];
 	char directory[PATH_MAX];
 	int x;
-	int error = 0;
-	int save = 0;
 	char buf[PATH_MAX + (NAME_MAX * 2) + 4];
+	int y = 0;
 
-	if(!curr_stats.num_yanked_files)
+	if (curr_stats.use_register && curr_stats.register_saved)
+		return put_files_from_register(view);
+
+	if (!curr_stats.num_yanked_files)
 		return 0;
 
-	if(!strcmp(curr_stats.yanked_files_dir, cfg.trash_dir))
+	if (!strcmp(curr_stats.yanked_files_dir, cfg.trash_dir))
 	{
 		snprintf(command, sizeof(command), "mv");
 		snprintf(directory, sizeof(directory), "%s",
@@ -323,22 +485,31 @@ put_files(FileView *view)
 
 	for(x = 0; x < curr_stats.num_yanked_files; x++)
 	{
-		snprintf(buf, sizeof(buf), "%s \"%s/%s\" %s",
-				command, directory, curr_stats.yanked_files[x],
-				view->curr_dir);
-		error = pipe_and_capture_errors(buf);
+		snprintf(buf, sizeof(buf), "%s/%s", directory, curr_stats.yanked_files[x]);
+		if (!access(buf, F_OK))
+		{
+			snprintf(buf, sizeof(buf), "%s \"%s/%s\" %s",
+					command, directory, curr_stats.yanked_files[x],
+					view->curr_dir);
+			if ( background_and_wait_for_errors(buf))
+				y++;
+		}
 	}
 
-	if(!error)
+	if (y)
 	{
-		snprintf(buf, sizeof(buf), " %d %s inserted", x,
-				x==1 ? "file" : "files");
-		save = 1;
+		snprintf(buf, sizeof(buf), " %d %s inserted", y,
+				y==1 ? "file" : "files");
+
+		load_dir_list(view, 0);
+		moveto_list_pos(view, view->curr_line);
+
+		status_bar_message(buf);
+
+		return 1;
 	}
-	load_dir_list(view, 0);
-	moveto_list_pos(view, view->curr_line);
-	status_bar_message(buf);
-	return save;
+
+	return 0;
 }
 
 void
@@ -402,13 +573,16 @@ change_window(FileView **view)
 	switch_views();
 	*view = curr_view;
 
-	wattroff(other_view->title, A_BOLD);
-	wattroff(other_view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
-	mvwaddstr(other_view->win, other_view->curr_line, 0, "*");
-	erase_current_line_bar(other_view);
-	werase(other_view->title);
-	wprintw(other_view->title, other_view->curr_dir);
-	wnoutrefresh(other_view->title);
+	if (curr_stats.number_of_windows != 1)
+	{
+		wattroff(other_view->title, A_BOLD);
+		wattroff(other_view->win, COLOR_PAIR(CURR_LINE_COLOR) | A_BOLD);
+		mvwaddstr(other_view->win, other_view->curr_line, 0, "*");
+		erase_current_line_bar(other_view);
+		werase(other_view->title);
+		wprintw(other_view->title, other_view->curr_dir);
+		wnoutrefresh(other_view->title);
+	}
 
 	wattron(curr_view->title, A_BOLD);
 	werase(curr_view->title);
@@ -420,12 +594,15 @@ change_window(FileView **view)
 
 	change_directory(curr_view, curr_view->curr_dir);
 
-	if(cfg.show_full)
+	if (curr_stats.number_of_windows == 1)
 		load_dir_list(curr_view, 1);
 
 	moveto_list_pos(curr_view, curr_view->list_pos);
 	werase(status_bar);
 	wnoutrefresh(status_bar);
+	
+	if (curr_stats.number_of_windows == 1)
+		update_all_windows();
 }
 
 static void
@@ -476,41 +653,96 @@ filter_selected_files(FileView *view)
 void
 update_all_windows(void)
 {
-	touchwin(lborder);
-	touchwin(lwin.title);
-	touchwin(lwin.win);
-	touchwin(mborder);
-	touchwin(rwin.title);
-	touchwin(rwin.win);
-	touchwin(stat_win);
-	touchwin(status_bar);
-	touchwin(pos_win);
-	touchwin(num_win);
-	touchwin(rborder);
+	/* In One window view */
+	if (curr_stats.number_of_windows == 1)
+	{
+		if (curr_view == &lwin)
+		{
+			touchwin(lwin.title);
+			touchwin(lwin.win);
+			touchwin(lborder);
+			touchwin(stat_win);
+			touchwin(status_bar);
+			touchwin(pos_win);
+			touchwin(num_win);
+			touchwin(rborder);
 
-	/*
-	 * redrawwin() shouldn't be needed.  But without it there is a 
-	 * lot of flickering when redrawing the windows?
-	 */
+			/*
+			 * redrawwin() shouldn't be needed.  But without it there is a 
+			 * lot of flickering when redrawing the windows?
+			 */
 
-	redrawwin(lborder);
-	redrawwin(lwin.title);
-	redrawwin(lwin.win);
-	redrawwin(mborder);
-	redrawwin(rwin.title);
-	redrawwin(rwin.win);
-	redrawwin(stat_win);
-	redrawwin(status_bar);
-	redrawwin(pos_win);
-	redrawwin(num_win);
-	redrawwin(rborder);
+			redrawwin(lborder);
+			redrawwin(stat_win);
+			redrawwin(status_bar);
+			redrawwin(pos_win);
+			redrawwin(lwin.title);
+			redrawwin(lwin.win);
+			redrawwin(num_win);
+			redrawwin(rborder);
+
+			wnoutrefresh(lwin.title);
+			wnoutrefresh(lwin.win);
+		}
+		else
+		{
+			touchwin(rwin.title);
+			touchwin(rwin.win);
+			touchwin(lborder);
+			touchwin(stat_win);
+			touchwin(status_bar);
+			touchwin(pos_win);
+			touchwin(num_win);
+			touchwin(rborder);
+
+			redrawwin(rwin.title);
+			redrawwin(rwin.win);
+			redrawwin(lborder);
+			redrawwin(stat_win);
+			redrawwin(status_bar);
+			redrawwin(pos_win);
+			redrawwin(num_win);
+			redrawwin(rborder);
+
+			wnoutrefresh(rwin.title);
+			wnoutrefresh(rwin.win);
+		}
+	}
+	/* Two Pane View */
+	else
+	{
+		touchwin(lwin.title);
+		touchwin(lwin.win);
+		touchwin(mborder);
+		touchwin(rwin.title);
+		touchwin(rwin.win);
+		touchwin(lborder);
+		touchwin(stat_win);
+		touchwin(status_bar);
+		touchwin(pos_win);
+		touchwin(num_win);
+		touchwin(rborder);
+
+		redrawwin(lwin.title);
+		redrawwin(lwin.win);
+		redrawwin(mborder);
+		redrawwin(rwin.title);
+		redrawwin(rwin.win);
+		redrawwin(lborder);
+		redrawwin(stat_win);
+		redrawwin(status_bar);
+		redrawwin(pos_win);
+		redrawwin(num_win);
+		redrawwin(rborder);
+
+		wnoutrefresh(lwin.title);
+		wnoutrefresh(lwin.win);
+		wnoutrefresh(mborder);
+		wnoutrefresh(rwin.title);
+		wnoutrefresh(rwin.win);
+	}
 
 	wnoutrefresh(lborder);
-	wnoutrefresh(lwin.title);
-	wnoutrefresh(lwin.win);
-	wnoutrefresh(mborder);
-	wnoutrefresh(rwin.title);
-	wnoutrefresh(rwin.win);
 	wnoutrefresh(stat_win);
 	wnoutrefresh(status_bar);
 	wnoutrefresh(pos_win);
@@ -520,29 +752,10 @@ update_all_windows(void)
 	doupdate();
 }
 
-static void
-reload_window(FileView *view)
-{
-	struct stat s;
 
-	stat(view->curr_dir, &s);
-	if(view != curr_view)
-		change_directory(view, view->curr_dir);
-	load_dir_list(view, 1);
-	view->dir_mtime = s.st_mtime;
-	if(view != curr_view)
-	{
-		change_directory(curr_view, curr_view->curr_dir);
-		mvwaddstr(view->win, view->curr_line, 0, "*");
-		wrefresh(view->win);
-	}
-	else
-		moveto_list_pos(view, view->list_pos);
-
-}
 
 /*
- * Main Loop 
+ *  Main Loop 
  * 	Everything is driven from this function with the exception of 
  * 	signals which are handled in signals.c
  */
@@ -554,9 +767,9 @@ main_key_press_cb(FileView *view)
 	int save_count = 0;
 	int count = 0;
 	int key = 0;
-	int save_msg = 0;
 	char count_buf[64] = "";
 	char status_buf[64] = "";
+	int save_reg = 0;
 
 	curs_set(0);
 
@@ -566,12 +779,9 @@ main_key_press_cb(FileView *view)
 	wtimeout(curr_view->win, 1000);
 	wtimeout(other_view->win, 1000);
 
-	if(cfg.show_full)
-		show_full_file_properties(view);
-	else
-		update_stat_window(view);
+	update_stat_window(view);
 
-	if(view->selected_files)
+	if (view->selected_files)
 	{
 		snprintf(status_buf, sizeof(status_buf), "%d %s Selected",
 				view->selected_files, view->selected_files == 1 ? "File" :
@@ -579,36 +789,63 @@ main_key_press_cb(FileView *view)
 		status_bar_message(status_buf);
 	}
 
-	while(!done)
+	while (!done)
 	{
-		if(curr_stats.freeze)
+		if (curr_stats.freeze)
 			continue;
 
-		if(curr_stats.reload_other_window)
-		{
-			reload_window(other_view);
-			curr_stats.reload_other_window = 0;
-		}
-		if(curr_stats.reload_current_window)
-		{
-			reload_window(curr_view);
-			curr_stats.reload_current_window = 0;
-			if(!save_msg)
-				clean_status_bar(view);
-		}
+		/* Everything from here to if (key == ERR) gets called once a second */
 
-		save_msg = 0;
+		check_if_filelists_have_changed(view);
+		check_background_jobs();
+
+		if (!curr_stats.save_msg)
+		{
+			clean_status_bar(view);
+			wrefresh(status_bar);
+		}
 
 		/* This waits for 1 second then skips if no keypress. */
 		key = wgetch(view->win);
 
-		if(key == ERR)
+		if (key == ERR)
 			continue;
 
+		/* This point down gets called only when a key is actually pressed */
 
-		/* ascii 0 - 9 */
-		if((key > 47) && (key < 58))
+		curr_stats.save_msg = 0;
+
+		if (curr_stats.use_register && !curr_stats.register_saved)
 		{
+			if (is_valid_register(key))
+			{
+				curr_stats.curr_register = key;
+				curr_stats.register_saved = 1;
+				save_reg = 1;
+				continue;
+			}
+			else
+			{
+				status_bar_message("Invalid Register Key");
+				curr_stats.save_msg = 1;
+				wrefresh(status_bar);
+				curr_stats.use_register = 0;
+				curr_stats.curr_register = -1;
+				curr_stats.register_saved = 0;
+				save_reg = 0;
+				continue;
+			}
+		}
+		else if((key > 47) && (key < 58)) /* ascii 0 - 9 */
+		{
+			if (count > 62)
+			{
+				show_error_msg(" Number is too large ", 
+						"Vifm cannot handle that large of a number as a count. ");
+				clear_num_window();
+				continue;
+			}
+
 			count_buf[count] = key;
 			count++;
 			count_buf[count] = '\0';
@@ -620,6 +857,9 @@ main_key_press_cb(FileView *view)
 	  
 		switch(key)
 		{
+			case '"': /* "register */
+				curr_stats.use_register = 1;
+				break;
 			case 2: /* ascii Ctrl B */
 			case KEY_PPAGE:
 				view->list_pos = view->list_pos - view->window_rows;
@@ -644,19 +884,8 @@ main_key_press_cb(FileView *view)
 				moveto_list_pos(view, view->list_pos);
 				break;
 			case 7: /* ascii Ctrl G */
-				if(cfg.show_full)
-				{
-					change_directory(other_view, other_view->curr_dir);
-					load_dir_list(other_view, 1);
-					reset_last_char = 1;
-					cfg.show_full = 0;
-					change_directory(curr_view, curr_view->curr_dir);
-					wnoutrefresh(other_view->win);
-				}
-				else
-				{
-					cfg.show_full = 1;
-				}
+				if(!curr_stats.show_full)
+					curr_stats.show_full = 1;
 				break;
 			case 9: /* ascii Tab */
 			case 32:  /* ascii Spacebar */
@@ -683,17 +912,18 @@ main_key_press_cb(FileView *view)
 				}
 				break;
 			case '.': /* repeat last change */
+				repeat_last_command(view);
 				break;
 			case ':': /* command */
-				save_msg = get_command(view, GET_COMMAND, NULL);
+				curr_stats.save_msg = get_command(view, GET_COMMAND, NULL);
 				break;
 			case '/': /* search */
-				save_msg = get_command(view, GET_SEARCH_PATTERN, NULL);
+				curr_stats.save_msg = get_command(view, GET_SEARCH_PATTERN, NULL);
 				break;
 			case '?': /* search backwards */
 				break;
 			case '\'': /* mark */
-				save_msg = get_bookmark(view);
+				curr_stats.save_msg = get_bookmark(view);
 				break;
 			case '%': /* Jump to percent of file. */
 				if(count)
@@ -706,7 +936,6 @@ main_key_press_cb(FileView *view)
 				break;
 			case 'G': /* Jump to bottom of list. */
 				{
-
 					if(count)
 						moveto_list_pos(view, atoi(count_buf) -1);
 					else
@@ -888,7 +1117,7 @@ main_key_press_cb(FileView *view)
 				reset_last_char = 1;
 				break;
 			case 'p': /* put files */
-				save_msg = put_files(view);
+				curr_stats.save_msg = put_files(view);
 				break;
 			case 's': /* tmp shellout **** This should be done with key mapping */
 				shellout(NULL, 0);
@@ -899,7 +1128,7 @@ main_key_press_cb(FileView *view)
 				/* tbrown */
 			case 'V':
 			case 'v': /* Visual selection of files. */
-				save_msg = start_visual_mode(view);
+				curr_stats.save_msg = start_visual_mode(view);
 				break;
 			case 'w': /* cw change word */
 				{
@@ -911,7 +1140,7 @@ main_key_press_cb(FileView *view)
 			case 'Y': /* Y yank file */
 					yank_files(view, count, count_buf);
 					reset_last_char++;
-					save_msg = 1;
+					curr_stats.save_msg = 1;
 					break;
 
 			case 'y': /* yy yank file */
@@ -920,10 +1149,14 @@ main_key_press_cb(FileView *view)
 					{
 						yank_files(view, count, count_buf);
 						reset_last_char++;
-						save_msg = 1;
+						curr_stats.save_msg = 1;
+						save_reg = 0;
 					}
 					else
+					{
 						update_num_window("y");
+						save_reg = 1;
+					}
 					save_count = 1;
 				}
 				break;
@@ -949,7 +1182,7 @@ main_key_press_cb(FileView *view)
 			count = 0;
 		}
 
-		if(cfg.show_full)
+		if(curr_stats.show_full)
 			show_full_file_properties(view);
 		else
 			update_stat_window(view);
@@ -963,12 +1196,23 @@ main_key_press_cb(FileView *view)
 						view->selected_files, view->selected_files == 1 ? "File" :
 						"Files");
 				status_bar_message(status_buf);
+				curr_stats.save_msg = 1;
 			}
 		}
 
-		else if(!save_msg)
+		else if(!curr_stats.save_msg)
 			clean_status_bar(view);
 
+		if (curr_stats.use_register && curr_stats.register_saved)
+		{
+			if (!save_reg)
+			{
+				curr_stats.use_register = 0;
+				curr_stats.curr_register = -1;
+				curr_stats.register_saved = 0;
+			}
+		}
+
 		if(curr_stats.need_redraw)
 			redraw_window();
 
