@@ -18,5 +18,3 @@
 
 
 void setup_signals(void);
-void switch_views(void);
-void start_timed_update_thread(void);
