@@ -26,7 +26,6 @@ typedef struct
 	char **yanked_files;
 	int need_redraw;
 	volatile int freeze;
-	volatile int block_timer_thread;
 	volatile int getting_input;
 	volatile int menu;
 	int redraw_menu;
@@ -36,8 +35,12 @@ typedef struct
 	int is_console;
 	time_t config_file_mtime;
 	int search;
-	int reload_current_window;
-	int reload_other_window;
+	int save_msg;
+	int use_register;
+	int curr_register;
+	int register_saved;
+	int number_of_windows;
+	int show_full;
 }Status;
 
 extern Status curr_stats;
